==> mingw-w64-libconfini-1.13.0-1.src.tar.gz: libconfini-1.13.0/examples/utilities/make_strarray.h <==
/*  examples/utilities/make_strarray.h  */

#include <stdlib.h>
#include <string.h>
#include <confini.h>

/*

  @brief    Converts a stringified INI array to a new array of strings
  @param    str             The stringified array (it cannot be `NULL`)
  @param    len             The length of the stringified array
  @param    delimiter       The delimiter between the array members -- if zero
                            (see #INI_ANY_SPACE), any space is delimiter
                            (`/(?:\\(?:\n\r?|\r\n?)|[\t \v\f])+/`)
  @param    format          The format of the INI file
  @param    dest_arrlen     The variable where the length of the array (in
                            number of members) will be saved
  @return   A new array of char pointers

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~{.c}
  size_t my_arrlen;
  char ** my_array = make_strarray(
    this->value,
    this->v_len,
    INI_ANY_SPACE,
    this->format,
    &my_arrlen
  );

  // Do something with `my_array`...

  free(my_array);
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  The returned array must be freed afterwards.

*/
static char ** make_strarray (
  const char * const str,
  const size_t len,
  const char delimiter,
  const IniFormat format,
  size_t * const dest_arrlen
) {

  *dest_arrlen = ini_array_get_length(str, delimiter, format);

  char ** const
    newarr  =   *dest_arrlen ?
                  (char **) malloc(*dest_arrlen * sizeof(char *) + len + 1)
                :
                  NULL;


  if (!newarr) {

    return NULL;

  }

  memcpy(newarr + *dest_arrlen, str, len + 1);

  char * iter = (char *) ((char **) newarr + *dest_arrlen);

  for (size_t idx = 0; idx < *dest_arrlen; idx++) {

    newarr[idx] = ini_array_release(&iter, delimiter, format);
    ini_string_parse(newarr[idx], format);

  }

  return newarr;

}

==> mingw-w64-libconfini-1.15.0-1.src.tar.gz: libconfini-1.15.0/examples/utilities/make_strarray.h <==
/*  examples/utilities/make_strarray.h  */

#include <stdlib.h>
#include <string.h>
#include <confini.h>

/**

  @brief    Convert a stringified INI array to a new array of strings
  @param    dest_arrlen     The variable where the length of the array (in
                            number of members) will be saved (cannot be
                            `NULL`)
  @param    str             The stringified array (it cannot be `NULL`)
  @param    len             The length of the stringified array in bytes
  @param    delimiter       The delimiter between the array members -- if zero
                            (see #INI_ANY_SPACE), any space is delimiter
                            (`/(?:\\(?:\n\r?|\r\n?)|[\t \v\f])+/`)
  @param    format          The format of the INI file
  @return   A new array of char pointers

  The returned address (and **only that**) must be freed afterwards.

  Example usage:

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~{.c}
  size_t my_arrlen;

  char ** my_array = make_strarray(
    &my_arrlen,
    this->value,
    this->v_len,
    INI_ANY_SPACE,
    this->format
  );

  if (my_arrlen && !my_array) {
    fprintf(stderr, "malloc() failed");
    exit(1);
  }

  // Do something with `my_array`...

  if (my_array) {
    free(my_array);
  }
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

**/
static char ** make_strarray (
  size_t * const dest_arrlen,
  const char * const str,
  const size_t len,
  const char delimiter,
  const IniFormat format
) {

  *dest_arrlen = ini_array_get_length(str, delimiter, format);

  char ** const
    newarr  =   *dest_arrlen ?
                  (char **) malloc(*dest_arrlen * sizeof(char *) + len + 1)
                :
                  NULL;

  if (!newarr) {
    return NULL;
  }

  char * remnant = (char *) ((char **) newarr + *dest_arrlen);
  memcpy(remnant, str, len + 1);
  for (size_t idx = 0; idx < *dest_arrlen; idx++) {
    newarr[idx] = ini_array_release(&remnant, delimiter, format);
    ini_string_parse(newarr[idx], format);
  }

  return newarr;

}
